/*
   Example command:
   00 01 23 15 ff - set time  23:15
     respond - Set time OK
   00 00 ff       - get time
     respond - Time is 23:15:00
*/

// #define  ENABLE_BIT_DEFINITIONS
#include <io2313.h>
#include <inavr.h>
#include <pgmspace.h>

// clock defenition
#define HR        0x85       // hour register read
#define MR        0x83       // minute register read
#define SR        0x81       // second register read

#define HW        0x84       // hour register write
#define MW        0x82       // minute register write
#define SW        0x80       // second register write

#define CW        0x8E       // control register write
#define TCW       0x90       // TRICKLE CHARGER write

#define PRE       0x00       // preambule
#define EOT       0xFF       // end of transmition

#define CMD_GETT  0x00
#define CMD_SETT  0x01

#define SIZE_RXB  10
#define SIZE_TXB  20

#define SCLK      PORTB_Bit4
#define IOR       PINB_Bit5
#define IOW       PORTB_Bit5
#define RST       PORTB_Bit6

#define RST_MC    PORTD_Bit2

char rx_buf[SIZE_RXB];
char tx_buf[SIZE_TXB];

char* prx;
char* ptx = tx_buf;
char flag_rx=0;
char flag_tx=0;
signed char state = -2;

__flash char msg1[] = "Set time OK";
__flash char msg2[] = "Time is XX:XX:XX";
__flash char msg3[] = "Unknown command";
__eeprom char clock_set = 0;

char data_read(char addr);
void data_write(char addr, char data);
void set_clock(char hour, char min);
void res_mc();
//**************************************
//  RXC -    UART 
//     

#pragma vector=UART_RX_vect
__interrupt void my_RX_routine()
{
   char rec_byte = UDR;
   if(flag_rx == 0)
   {
      switch(state){
        case 0:
              if(rec_byte == EOT)
              {
                 flag_rx = 1;
                 state = -2;
              }
              else
              {
                 *prx = rec_byte;
                 prx++;
              }
              break;
        default:
              {
                if(rec_byte == PRE)
                {
                  prx = rx_buf;
                  state = 0;
                }
                else
                  state = -2;
              }
      }
   }
}
//**************************************
//  UDRE - ,   UDR,
//     ,  
//      .
#pragma vector=UART_UDRE_vect
__interrupt void my_UDRE_routine()
{
   UDR = *ptx;
   ptx++;
   if(*ptx == 0)       //    
   {
      UCR &= ~(1<<UDRIE);   // clear bit UDRIE
      ptx = tx_buf;
   }
}
//*************************************
//*************************************
int main(void)
{
   char hour;
   char min;
   char sec;
   #define Fosc 7372800.0
   #define BOUD 2400
   // Communication Parameters: 8 Data, 1 Stop, No Parity
   // USART Mode: Asynchronous
   // Speed = 2400   
   UCR = (1<<RXCIE) | (1<<RXEN) | (1<<TXEN) ;
   UBRR = Fosc / (16.0 * BOUD ) - 1;
   RST_MC = 0;
   __enable_interrupt();
   do{
        if(flag_rx == 1)
        {
           switch(rx_buf[0]){
            case CMD_GETT:
                           strcpy_P(tx_buf, msg2);
                           hour = data_read(HR);
                           min = data_read(MR);
                           sec = data_read(SR);
                           tx_buf[8] = ((hour & 0x30) >> 4) + 0x30;
                           tx_buf[9] = (hour & 0x0F) + 0x30;
                           tx_buf[11] = ((min & 0x70) >> 4) + 0x30;
                           tx_buf[12] = (min & 0x0F) + 0x30;
                           tx_buf[14] = ((sec & 0x70) >> 4) + 0x30;
                           tx_buf[15] = (sec & 0x0F) + 0x30;
                           break;
            case CMD_SETT:
                           hour = rx_buf[1];
                           min = rx_buf[2];
                           set_clock(hour, min);
                           strcpy_P(tx_buf, msg1);
                           clock_set = 1;
                           break;
            default:
                           strcpy_P(tx_buf, msg3);

           }
           flag_rx = 0;
           UCR |= 1<<UDRIE;           
        }
        if(clock_set == 1)
        {
            hour = data_read(HR);
            min = data_read(MR);
            if( ( min == 0) && ( (hour & 0x3f) == 0) ) //  0 
            {
               DDRD = 1<<DDD2;
               //  400 
               __delay_cycles(2949120);           
               DDRD &= ~(1<<DDD2);  
               //   20c
               __delay_cycles(147456000);
               data_write(SW, 0x00);   // set sec to 0
               //   1,5
               __delay_cycles(663552000);
            }
        }
   }while(1);
}
//*************************************
//      RTC
char data_read(char addr)
{
   char data=0;
   char i=8;
   DDRB=0x30;
   //    
   RST = 1;
   do {
         IOW = addr;
         addr>>=1;
         SCLK = 1;
         SCLK = 0;
   }while(--i>0);
   IOW = 0;
   DDRB_Bit5 = 0;
   i=8;
   do {
         data >>= 1;
         if(IOR == 1)
           data |= 0x80;
         SCLK = 1;
         SCLK = 0;
   }while(--i>0);
   RST = 0;
   DDRB = 0;
   return data;
}
//*************************************
void data_write(char addr, char data)
{
   char i=8;
   DDRB=0x30;
   RST = 1;
   do {
         IOW = addr;
         addr>>=1;
         SCLK = 1;
         SCLK = 0;
   }while(--i>0);
   i=8;
   do {
         IOW = data;
         data >>= 1;
         SCLK = 1;
         SCLK = 0;
   }while(--i>0);
   RST = 0;
   DDRB = 0;
}
//*************************************
void set_clock(char hour, char min)
{
   data_write(CW, 0x00);   // WP bit should be cleared before attempting to write to the device.
   //data_write(TCW, 0xAB); // write TRICKLE CHARGER, 8K, 2 DIODES

   data_write(HW, hour);   // set hour
   data_write(MW, min);    // set min
   data_write(SW, 0x00);   // set sec(start time)
}
//*************************************



